//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Realspace/RealSpaceCanvas.h
//! @brief     Defines class RealSpaceCanvas
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_REALSPACE_REALSPACECANVAS_H
#define BORNAGAIN_GUI_VIEW_REALSPACE_REALSPACECANVAS_H

#include <QWidget>
#include <memory>
#include <optional>
#include <variant>

class CautionSign;
class MaterialModel;
class MultiLayerItem;
class LayerItem;
class ParticleLayoutItem;
class ItemWithParticles;

namespace GUI::RealSpace {
class Canvas;
class Model;
} // namespace GUI::RealSpace

//! Size and thickness information of layers
struct SceneGeometry {
    double layerSize = 100.0;
    double topOrBottomLayerThickness = 25.0;
    double layerMinimumThickness = 2.0;
};

//! Provides 3D object generation for RealSpace presentation.
class RealSpaceCanvas : public QWidget {
    Q_OBJECT

public:
    //! Defines the item types which can be the "current item"
    using ItemForRealSpace =
        std::variant<MultiLayerItem*, LayerItem*, ParticleLayoutItem*, ItemWithParticles*>;

    RealSpaceCanvas(const MaterialModel* materialModel, QWidget* parent = nullptr);

    void setCurrentItem(ItemForRealSpace item);
    std::optional<ItemForRealSpace> currentItem() const;
    void defaultView();
    void sideView();
    void topView();
    void changeLayerSize(double layerSizeChangeScale);
    void savePicture();
    void resetScene();
    void updateScene();

protected:
    void showEvent(QShowEvent*) override;

private:
    void savePicture(const QPixmap& pixmap);

    const MaterialModel* m_materialModel;
    GUI::RealSpace::Canvas* m_view;
    std::unique_ptr<GUI::RealSpace::Model> m_realSpaceModel;
    SceneGeometry m_sceneGeometry;
    CautionSign* m_cautionSign;
    std::optional<ItemForRealSpace> m_currentItem;
    bool m_firstView;
};

#endif // BORNAGAIN_GUI_VIEW_REALSPACE_REALSPACECANVAS_H
